@@ -2,7 +2,7 @@
 
   meterec 
   Console based multi track digital peak meter and recorder for JACK
-  Copyright (C) 2009-2011 Fabrice Lebas
+  Copyright (C) 2009-2013 Fabrice Lebas
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -33,7 +33,7 @@
 #include "conf.h"
 #include "queue.h"
 
-#define RD_BUFF_LEN ((meterec->read_disk_buffer_process_pos - meterec->read_disk_buffer_thread_pos) & (DISK_SIZE-1))
+#define RD_BUFF_LEN ((meterec->read_disk_buffer_process_pos - meterec->read_disk_buffer_thread_pos) & (DBUF_SIZE-1))
 
 
 /******************************************************************************
@@ -81,10 +81,10 @@ SNDFILE* write_disk_open_fd(struct meterec_s *meterec) {
 	return out;
 }
 
-int writer_thread(void *d) {
-	unsigned int i, port, opos, track, thread_delay;
+void *writer_thread(void *d) {
+	unsigned int i, port, zbuff_pos, track, thread_delay;
 	SNDFILE *out;
-	float buf[BUF_SIZE * MAX_PORTS];
+	float buf[ZBUF_SIZE * MAX_PORTS];
 	struct meterec_s *meterec ;
 	
 	meterec = (struct meterec_s *)d ;
@@ -99,29 +99,29 @@ int writer_thread(void *d) {
 	out = write_disk_open_fd(meterec);
 	
 	if (!out)
-		return 1;
+		return (void*)1;
 	
 	/* Start writing the RT ringbuffer to disk */
 	meterec->record_sts = ONGOING ;
-	opos = 0;
+	zbuff_pos = 0;
 	while (meterec->record_sts) {
 		
 		for (i  = meterec->write_disk_buffer_thread_pos; 
-			i != meterec->write_disk_buffer_process_pos && opos < BUF_SIZE;
-			i  = (i + 1) & (DISK_SIZE - 1), opos++ ) {
+			i != meterec->write_disk_buffer_process_pos && zbuff_pos < ZBUF_SIZE;
+			i  = (i + 1) & (DBUF_SIZE - 1), zbuff_pos++ ) {
 			
 			track = 0;
 			for (port = 0; port < meterec->n_ports; port++) {
 				if (meterec->ports[port].record) {
-					buf[opos * meterec->n_tracks + track] = meterec->ports[port].write_disk_buffer[i]; 
+					buf[zbuff_pos * meterec->n_tracks + track] = meterec->ports[port].write_disk_buffer[i]; 
 					track++;
 				}
 			}
 		}
 		
-		if (opos == BUF_SIZE) {
-			sf_writef_float(out, buf, opos);
-			opos = 0;
+		if (zbuff_pos == ZBUF_SIZE) {
+			sf_writef_float(out, buf, zbuff_pos);
+			zbuff_pos = 0;
 		}
 		
 		meterec->write_disk_buffer_thread_pos = i;
@@ -145,7 +145,7 @@ int writer_thread(void *d) {
 		/* run until empty buffer after a stop requets */
 		if (meterec->record_sts == STOPING)
 			if ( meterec->write_disk_buffer_thread_pos == meterec->write_disk_buffer_process_pos ) {
-				sf_writef_float(out, buf, opos);
+				sf_writef_float(out, buf, zbuff_pos);
 				break;
 			}
 		
@@ -165,7 +165,7 @@ int writer_thread(void *d) {
 	
 	meterec->record_sts = OFF;
 	
-	return 0;
+	return (void*)0;
 }
 
 void read_disk_close_fd(struct meterec_s *meterec) {
@@ -197,7 +197,7 @@ void read_disk_open_fd(struct meterec_s *meterec) {
 			fprintf(meterec->fd_log,"Reader thread: Port %d does not have a take associated\n", port+1);
 			
 			/* rather fill buffer with 0's */
-			for (i=0; i<DISK_SIZE; i++) 
+			for (i=0; i<DBUF_SIZE; i++) 
 				meterec->ports[port].read_disk_buffer[i] = 0.0f ;
 			
 			continue;
@@ -209,7 +209,7 @@ void read_disk_open_fd(struct meterec_s *meterec) {
 			fprintf(meterec->fd_log,"Reader thread: Port %d beeing recorded in REC mode will not have a playback take associated\n", port+1);
 			
 			/* rather fill buffer with 0's */
-			for (i=0; i<DISK_SIZE; i++) 
+			for (i=0; i<DBUF_SIZE; i++) 
 				meterec->ports[port].read_disk_buffer[i] = 0.0f ;
 			
 			continue;
@@ -237,10 +237,10 @@ void read_disk_open_fd(struct meterec_s *meterec) {
 			/* allocate buffer space for this take */
 			fprintf(meterec->fd_log,"Reader thread: Allocating local buffer space %d*%d for take %d\n", 
 				meterec->takes[take].ntrack, 
-				BUF_SIZE, 
+				ZBUF_SIZE, 
 				take);
 			
-			meterec->takes[take].buf = calloc(BUF_SIZE*meterec->takes[take].ntrack, sizeof(float));
+			meterec->takes[take].buf = calloc(ZBUF_SIZE*meterec->takes[take].ntrack, sizeof(float));
 			
 		} 
 		else {
@@ -251,13 +251,23 @@ void read_disk_open_fd(struct meterec_s *meterec) {
 	
 }
 
-unsigned int fill_buffer(struct meterec_s *meterec, unsigned int *opos ) {
+unsigned int fill_buffer(struct meterec_s *meterec, unsigned int *zbuff_pos ) {
 	
-	unsigned int i,  port, take, track, ntrack=0, fill;
+	/* real read from disk thru libsndfile to fill 'buffer zero' then copy from
+	   'buffer zero' to 'disk buffer' that is the connection with jack process */
 	
-	/* lets fill local buffer only if previously emptied*/
-	if (*opos == 0) {
+	unsigned int rdbuff_pos, port, take, track, ntrack=0, fill;
 	
+	/* Leave right away if the process does not need any data (disk buffer full) */
+	if (meterec->read_disk_buffer_thread_pos == meterec->read_disk_buffer_process_pos)
+		return meterec->read_disk_buffer_thread_pos;
+	
+	/* lets fill local buffer only if previously emptied */
+	if (*zbuff_pos == 0) {
+	
+	#ifdef DEBUG_BUFF
+	fprintf(meterec->fd_log, "fill_buffer: Filling zero buffer -------------------------------\n");
+	#endif
 		/* load the local buffer */
 		for(take=1; take<meterec->n_takes+1; take++) {
 			
@@ -268,10 +278,10 @@ unsigned int fill_buffer(struct meterec_s *meterec, unsigned int *opos ) {
 			/* get the number of tracks in this take */
 			ntrack = meterec->takes[take].ntrack;
 			
-			fill = sf_read_float(meterec->takes[take].take_fd, meterec->takes[take].buf, (BUF_SIZE * ntrack) ); 
+			fill = sf_read_float(meterec->takes[take].take_fd, meterec->takes[take].buf, (ZBUF_SIZE * ntrack) ); 
 			
 			/* complete buffer with 0's if reached end of file */
-			for ( ; fill<(BUF_SIZE * ntrack); fill++) 
+			for ( ; fill<(ZBUF_SIZE * ntrack); fill++) 
 				meterec->takes[take].buf[fill] = 0.0f;
 			
 		}
@@ -280,10 +290,10 @@ unsigned int fill_buffer(struct meterec_s *meterec, unsigned int *opos ) {
 	
 	
 	
-	/* walk in the local buffer and copy it to each port buffers (demux)*/
-	for (i  = meterec->read_disk_buffer_thread_pos; 
-		i != meterec->read_disk_buffer_process_pos && *opos < BUF_SIZE;
-		i  = (i + 1) & (DISK_SIZE - 1), (*opos)++, meterec->disk.playhead++ ) {
+	/* walk in the zero buffer and copy it to each port buffers (demux)*/
+	for (rdbuff_pos  = meterec->read_disk_buffer_thread_pos; 
+		rdbuff_pos != meterec->read_disk_buffer_process_pos && *zbuff_pos < ZBUF_SIZE;
+		rdbuff_pos  = (rdbuff_pos + 1) & (DBUF_SIZE - 1), (*zbuff_pos)++, meterec->disk.playhead++ ) {
 		
 		for(take=1; take<meterec->n_takes+1; take++) {
 			
@@ -304,7 +314,7 @@ unsigned int fill_buffer(struct meterec_s *meterec, unsigned int *opos ) {
 				if (meterec->ports[port].playback_take == take)
 					/* Only fill buffer if in playback, dub or overdub */
 					if (meterec->ports[port].record != REC || !meterec->record_sts)
-						meterec->ports[port].read_disk_buffer[i] = meterec->takes[take].buf[*opos * ntrack + track] ;
+						meterec->ports[port].read_disk_buffer[rdbuff_pos] = meterec->takes[take].buf[*zbuff_pos * ntrack + track] ;
 				
 			}
 		
@@ -312,32 +322,52 @@ unsigned int fill_buffer(struct meterec_s *meterec, unsigned int *opos ) {
 	
 	}
 	
-	if (*opos == BUF_SIZE) 
-		*opos = 0;
+	if (*zbuff_pos == ZBUF_SIZE) 
+		*zbuff_pos = 0;
 	
-	return i;
+	#ifdef DEBUG_BUFF
+	fprintf(meterec->fd_log, "fill_buffer: zbuff_pos %4X/%4X | rdbuff_pos %5X/%5X | thread_pos %5X/%5X | process_pos %5X/%5X\n", 
+		*zbuff_pos, ZBUF_SIZE, 
+		rdbuff_pos, DBUF_SIZE, 
+		meterec->read_disk_buffer_thread_pos, DBUF_SIZE,
+		meterec->read_disk_buffer_process_pos, DBUF_SIZE);
+	#endif
+	
+	return rdbuff_pos;
 	
 }
 
 void read_disk_seek(struct meterec_s *meterec, unsigned int seek) {
 	
 	unsigned int take;
+	sf_count_t reached;
 	
 	for(take=1; take<meterec->n_takes+1; take++) {
 	
 		/* check if track is used */
 		if (meterec->takes[take].take_fd == NULL)
 			continue;
-			
-		sf_seek(meterec->takes[take].take_fd, seek, SEEK_SET);	
+		
+		#ifdef DEBUG_SEEK
+		fprintf(meterec->fd_log, "read_disk_seek: seek take %d at position %d 0x%X (%.3f).\n", take, seek, seek, (float)seek/meterec->jack.sample_rate);
+		#endif
+		
+		reached = sf_seek(meterec->takes[take].take_fd, seek, SEEK_SET);
+		
+		if (seek - reached) {
+			#ifdef DEBUG_SEEK
+			fprintf(meterec->fd_log, "read_disk_seek: failed (seek=%d reached=%d)", seek, reached);
+			#endif
+			sf_seek(meterec->takes[take].take_fd, 0, SEEK_END);
+		}
 	}
 
 }
 
 
-int reader_thread(void *d)
+void *reader_thread(void *d)
 {
-	unsigned int i, opos, thread_delay, may_loop;
+	unsigned int rdbuff_pos, zbuff_pos, thread_delay, may_loop;
 	struct event_s *event, *event_kill;
 	struct meterec_s *meterec ;
 	
@@ -349,9 +379,9 @@ int reader_thread(void *d)
 	
 	thread_delay = set_thread_delay(meterec);
 	
-	/* empty buffer ( reposition thread position in order to refill where process will first read) */
+	/* empty buffer (reposition thread position in order to refill where process will first read) */
 	meterec->read_disk_buffer_thread_pos  = (meterec->read_disk_buffer_process_pos + 1);
-	meterec->read_disk_buffer_thread_pos &= (DISK_SIZE - 1);
+	meterec->read_disk_buffer_thread_pos &= (DBUF_SIZE - 1);
 	
 	/* open all files needed for this playback */
 	read_disk_open_fd(meterec);
@@ -359,10 +389,10 @@ int reader_thread(void *d)
 	fprintf(meterec->fd_log,"Reader thread: Start reading files.\n");
 	
 	/* prefill buffer at once */
-	opos = 0;
+	zbuff_pos = 0;
 	while (meterec->read_disk_buffer_thread_pos != meterec->read_disk_buffer_process_pos) {
-		i = fill_buffer(meterec, &opos);
-		meterec->read_disk_buffer_thread_pos = i;
+		rdbuff_pos = fill_buffer(meterec, &zbuff_pos);
+		meterec->read_disk_buffer_thread_pos = rdbuff_pos;
 	}
 	
 	meterec->playback_sts=ONGOING;
@@ -370,6 +400,11 @@ int reader_thread(void *d)
 	/* Start reading disk to fill the RT ringbuffer */
 	while ( meterec->playback_cmd==START ) {
 		
+		#ifdef DEBUG_QUEUES
+		event_queue_print(meterec, LOG); 
+		#endif
+		
+		/* find ALL event types in DISK queue */
 		event = find_last_event(meterec, DISK, ALL);
 		
 		if (event) {
@@ -382,18 +417,20 @@ int reader_thread(void *d)
 				compute_takes_to_playback(meterec);
 				read_disk_open_fd(meterec);
 				
+				/* no break here */
+				
 			case SEEK:
 				
-				/* make sure we fill buffer away from where jack read to avoid having to wait filling ringbuffer*/
-				meterec->read_disk_buffer_thread_pos  = meterec->read_disk_buffer_process_pos - (DISK_SIZE/2);
-				meterec->read_disk_buffer_thread_pos &= (DISK_SIZE - 1);
+				/* make sure we fill buffer away from where jack read to avoid having to wait filling ringbuffer */
+				meterec->read_disk_buffer_thread_pos  = meterec->read_disk_buffer_process_pos - (DBUF_SIZE/2);
+				meterec->read_disk_buffer_thread_pos &= (DBUF_SIZE - 1);
 				
 				read_disk_seek(meterec, event->new_playhead);
 				
-				opos = 0;
+				zbuff_pos = 0;
 				
-				event->buffer_pos  = meterec->read_disk_buffer_thread_pos;
-				event->buffer_pos &= (DISK_SIZE - 1);
+				event->buffer_pos  = meterec->read_disk_buffer_thread_pos ;
+				event->buffer_pos &= (DBUF_SIZE - 1);
 				
 				meterec->disk.playhead = event->new_playhead;
 				
@@ -412,15 +449,16 @@ int reader_thread(void *d)
 				/* Is next loop event already in the buffer */
 				if (meterec->loop.enable) {
 					
-					if (meterec->disk.playhead > meterec->loop.high && meterec->jack.playhead < meterec->loop.high) {
-					
+					if (meterec->disk.playhead > meterec->loop.high &&
+						meterec->jack.playhead < meterec->loop.high) {
+						
 						/*we need to empty buffer up to the loop point */
 						meterec->read_disk_buffer_thread_pos -= meterec->loop.high;
 						meterec->read_disk_buffer_thread_pos += meterec->disk.playhead;
-						meterec->read_disk_buffer_thread_pos &= (DISK_SIZE - 1);
+						meterec->read_disk_buffer_thread_pos &= (DBUF_SIZE - 1);
 						
 						read_disk_seek(meterec, meterec->loop.low);
-						opos = 0;
+						zbuff_pos = 0;
 						
 						meterec->disk.playhead = meterec->loop.low;
 					}
@@ -457,42 +495,55 @@ int reader_thread(void *d)
 			if (meterec->disk.playhead < meterec->loop.high)
 				may_loop = 1;
 		
-		i = fill_buffer(meterec, &opos);
+		rdbuff_pos = fill_buffer(meterec, &zbuff_pos);
 		
 		if (may_loop)
 			if (meterec->disk.playhead >= meterec->loop.high) {
 				
 				read_disk_seek(meterec, meterec->loop.low);
-				opos = 0;
+				zbuff_pos = 0;
 				
-				i -= (meterec->disk.playhead - meterec->loop.high);
-				i &= (DISK_SIZE - 1);
+				rdbuff_pos -= (meterec->disk.playhead - meterec->loop.high);
+				rdbuff_pos &= (DBUF_SIZE - 1);
 				
 				meterec->disk.playhead = meterec->loop.low;
 				
 				pthread_mutex_lock(&meterec->event_mutex);
-				add_event(meterec, JACK, LOOP, meterec->loop.low, meterec->loop.high, i);
+				add_event(meterec, JACK, LOOP, meterec->loop.low, meterec->loop.high, rdbuff_pos);
 				pthread_mutex_unlock(&meterec->event_mutex);
 			}
 		
+		#ifdef DEBUG_QUEUES
+		event_queue_print(meterec, LOG); 
+		#endif
+		
 		event = find_first_event(meterec, PEND, ALL);
 		
 		if (event) 
 			switch (event->type) {
 				case SEEK:
-				
-					if (meterec->read_disk_buffer_thread_pos != i) 
+					
+					/* do not pass the event to jack process if we did not have data
+					   to fill the buffer */
+					if (meterec->read_disk_buffer_thread_pos != rdbuff_pos)
 						event->queue = JACK;
 					break;
 					
 				case LOCK:
-					if (meterec->disk.playhead > meterec->jack.playhead + BUF_SIZE)
+					/* be less reactive but more secure when changing take locks
+					   and only tell jack process to jump once we have a full 
+					   'buffer zero' adavance */
+					if (meterec->disk.playhead > meterec->jack.playhead + ZBUF_SIZE)
 						event->queue = JACK;
 					break;
 						
 			}
 		
-		meterec->read_disk_buffer_thread_pos = i;
+		meterec->read_disk_buffer_thread_pos = rdbuff_pos;
+		
+		#ifdef DEBUG_QUEUES
+		event_queue_print(meterec, LOG); 
+		#endif
 		
 		usleep(thread_delay);
 		
@@ -511,20 +562,20 @@ int reader_thread(void *d)
 
 float read_disk_buffer_level(struct meterec_s *meterec) {
 	float level;
-	level = (meterec->read_disk_buffer_process_pos - meterec->read_disk_buffer_thread_pos) & (DISK_SIZE-1);
-	return  (float)(level / DISK_SIZE);
+	level = (meterec->read_disk_buffer_process_pos - meterec->read_disk_buffer_thread_pos) & (DBUF_SIZE-1);
+	return  (float)(level / DBUF_SIZE);
 }
 
 float write_disk_buffer_level(struct meterec_s *meterec) {
 	float level;
-	level = (meterec->write_disk_buffer_process_pos - meterec->write_disk_buffer_thread_pos) & (DISK_SIZE-1);
-	return  (float)(level / DISK_SIZE);
+	level = (meterec->write_disk_buffer_process_pos - meterec->write_disk_buffer_thread_pos) & (DBUF_SIZE-1);
+	return  (float)(level / DBUF_SIZE);
 }
 
 unsigned int set_thread_delay(struct meterec_s *meterec) {
 	
 	/* How long should we wait to read 10 times faster than data goes away */
-	return 1000000ul * BUF_SIZE / meterec->jack.sample_rate / 10; 
+	return 1000000ul * ZBUF_SIZE / meterec->jack.sample_rate / 10; 
 	
 }
 
