@@ -2,7 +2,7 @@
 
   meterec 
   Console based multi track digital peak meter and recorder for JACK
-  Copyright (C) 2009-2011 Fabrice Lebas
+  Copyright (C) 2009-2013 Fabrice Lebas
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -20,8 +20,8 @@
 
 */
 
-int writer_thread(void *d);
-int reader_thread(void *d);
+void *writer_thread(void *d);
+void *reader_thread(void *d);
 float read_disk_buffer_level(struct meterec_s *meterec);
 float write_disk_buffer_level(struct meterec_s *meterec);
 unsigned int set_thread_delay(struct meterec_s *meterec);
