@@ -2,7 +2,7 @@
 
   meterec 
   Console based multi track digital peak meter and recorder for JACK
-  Copyright (C) 2009-2011 Fabrice Lebas
+  Copyright (C) 2009-2013 Fabrice Lebas
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -20,4 +20,4 @@
 
 */
 
-int keyboard_thread(void *arg);
+void *keyboard_thread(void *arg);
