@@ -2,7 +2,7 @@
 
   meterec.c 
   Console based multi track digital peak meter and recorder for JACK
-  Copyright (C) 2009 2010 2011 Fabrice Lebas
+  Copyright (C) 2009-2013 Fabrice Lebas
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -200,8 +200,14 @@ void read_peak(float bias) {
 int set_loop(struct meterec_s *meterec, unsigned int loophead) {
 	
 	if (meterec->loop.low == MAX_UINT) {
-		meterec->loop.low = loophead;
-		return 0;
+		if (meterec->loop.high == MAX_UINT) {
+			meterec->loop.low = loophead;
+			return 0;
+		}
+		if (loophead > meterec->loop.high) {
+			meterec->loop.low = meterec->loop.high;
+			meterec->loop.high = loophead;
+		}
 	}
 	else if (loophead > meterec->loop.low) {
 		meterec->loop.high = loophead;
@@ -220,11 +226,15 @@ int set_loop(struct meterec_s *meterec, unsigned int loophead) {
 	return 1;
 }
 
-void clr_loop(struct meterec_s *meterec) {
+void clr_loop(struct meterec_s *meterec, unsigned int bound) {
 	
 	meterec->loop.enable = 0;
-	meterec->loop.low = MAX_UINT;
-	meterec->loop.high = MAX_UINT;
+	
+	if (bound & BOUND_LOW)
+		meterec->loop.low = MAX_UINT;
+	
+	if (bound & BOUND_HIGH)
+		meterec->loop.high = MAX_UINT;
 	
 	pthread_mutex_lock( &meterec->event_mutex );
 	add_event(meterec, DISK, LOOP, MAX_UINT, MAX_UINT, MAX_UINT);
@@ -760,7 +770,7 @@ static int process_jack_data(jack_nframes_t nframes, void *arg) {
 				
 				/* if we seek because of a file re-open, compensate for what played since re-open request */
 				meterec->read_disk_buffer_process_pos += (meterec->jack.playhead  - event->new_playhead);
-				meterec->read_disk_buffer_process_pos &= (DISK_SIZE - 1);
+				meterec->read_disk_buffer_process_pos &= (DBUF_SIZE - 1);
 				
 				/*
 				event_print(meterec, LOG, event);
@@ -827,7 +837,7 @@ static int process_jack_data(jack_nframes_t nframes, void *arg) {
 					out[i] = meterec->ports[port].read_disk_buffer[read_pos];
 				
 				/* update buffer pointer */
-				read_pos = (read_pos + 1) & (DISK_SIZE - 1);
+				read_pos = (read_pos + 1) & (DBUF_SIZE - 1);
 				
 				/* compute peak of input (recordable) data*/
 				s = fabs(in[i] * 1.0f) ;
@@ -853,7 +863,7 @@ static int process_jack_data(jack_nframes_t nframes, void *arg) {
 					meterec->ports[port].write_disk_buffer[write_pos] = in[i];
 				
 				/* update buffer pointer */
-				write_pos = (write_pos + 1) & (DISK_SIZE - 1);
+				write_pos = (write_pos + 1) & (DBUF_SIZE - 1);
 				
 				}
 			
@@ -883,13 +893,13 @@ static int process_jack_data(jack_nframes_t nframes, void *arg) {
 	if (playback_ongoing) {
 		
 		/* track buffer over/under flow -- needs rework */
-		remaining_read_disk_buffer = DISK_SIZE - ((meterec->read_disk_buffer_thread_pos-meterec->read_disk_buffer_process_pos) & (DISK_SIZE-1));
+		remaining_read_disk_buffer = DBUF_SIZE - ((meterec->read_disk_buffer_thread_pos-meterec->read_disk_buffer_process_pos) & (DBUF_SIZE-1));
 		
 		if (remaining_read_disk_buffer <= nframes)
 			meterec->read_disk_buffer_overflow++;
 		
 		/* positon read pointer to end of ringbuffer */
-		meterec->read_disk_buffer_process_pos = (meterec->read_disk_buffer_process_pos + nframes) & (DISK_SIZE - 1);
+		meterec->read_disk_buffer_process_pos = (meterec->read_disk_buffer_process_pos + nframes) & (DBUF_SIZE - 1);
 		
 		/* set new playhead position */
 		meterec->jack.playhead += nframes ;
@@ -907,13 +917,13 @@ static int process_jack_data(jack_nframes_t nframes, void *arg) {
 		if (record_ongoing) {
 			
 			/* track buffer over/under flow */
-			remaining_write_disk_buffer = DISK_SIZE - ((meterec->write_disk_buffer_process_pos-meterec->write_disk_buffer_thread_pos) & (DISK_SIZE-1));
+			remaining_write_disk_buffer = DBUF_SIZE - ((meterec->write_disk_buffer_process_pos-meterec->write_disk_buffer_thread_pos) & (DBUF_SIZE-1));
 			
 			if (remaining_write_disk_buffer <= nframes)
 				meterec->write_disk_buffer_overflow++;
 			
 			/* positon write pointer to end of ringbuffer*/
-			meterec->write_disk_buffer_process_pos = (meterec->write_disk_buffer_process_pos + nframes) & (DISK_SIZE - 1);
+			meterec->write_disk_buffer_process_pos = (meterec->write_disk_buffer_process_pos + nframes) & (DBUF_SIZE - 1);
 		
 		}
 	
@@ -935,7 +945,7 @@ void start_playback(struct meterec_s *meterec) {
 	
 	compute_takes_to_playback(meterec);
 	meterec->playback_cmd = START ;
-	pthread_create(&rd_dt, NULL, (void *)&reader_thread, (void *) meterec);
+	pthread_create(&rd_dt, NULL, reader_thread, (void *)meterec);
 
 }
 
@@ -944,7 +954,7 @@ void start_record(struct meterec_s *meterec) {
 	compute_tracks_to_record(meterec);
 	if (meterec->n_tracks) {
 		meterec->record_cmd = START;
-		pthread_create(&wr_dt, NULL, (void *)&writer_thread, (void *) meterec);
+		pthread_create(&wr_dt, NULL, writer_thread, (void *)meterec);
 	}
 
 }
@@ -1050,6 +1060,8 @@ static int usage( const char * progname ) {
 	fprintf(stderr, " <CTRL>F1-F12  use time index as loop boundary\n");
 	fprintf(stderr, "       +       use current time as loop boundary\n");
 	fprintf(stderr, "       -       clear loop boundaries\n");
+	fprintf(stderr, "       /       clear loop lower bound\n");
+	fprintf(stderr, "       *       clear loop upper bound\n");
 	fprintf(stderr, "       <HOME>  be kind, rewind\n");
 	fprintf(stderr, "       <TAB>   vu-meter view (special keys) ------------------------------------\n");
 	fprintf(stderr, "       =>      seek forward 5 seconds\n");
@@ -1060,8 +1072,8 @@ static int usage( const char * progname ) {
 	fprintf(stderr, "       l       lock/unlock selected track for playback\n");
 	fprintf(stderr, "       L       lock/unlock selected track for playback and clear all other locks for this port\n");
 	fprintf(stderr, "       a       lock/unlock selected take for playback\n");
-	fprintf(stderr, "       A       lock/unlock selected take for playback and clear all other locks in the session)\n");
-	fprintf(stderr, "       <TAB>   connections view (special keys)----------------------------------\n");
+	fprintf(stderr, "       A       lock/unlock selected take for playback and clear all other locks in the session\n");
+	fprintf(stderr, "       <TAB>   connections view (special keys) ---------------------------------\n");
 	fprintf(stderr, "       <= =>   select port column\n");
 	fprintf(stderr, "       c       connect ports\n");
 	fprintf(stderr, "       x       disconnect ports\n");
@@ -1207,7 +1219,7 @@ int main(int argc, char *argv[])
 	if (file_exists(meterec->conf_file)) {
 		load_conf(meterec);
 		if (meterec->connect_ports)
-			connect_all_ports(meterec);
+			connect_all_ports((void*)meterec);
 	} else {
 		load_setup(meterec);
 		load_session(meterec);
@@ -1246,7 +1258,7 @@ int main(int argc, char *argv[])
 	curs_set(0);
 	start_color();
 	
-	// choose our color pairs
+	/* choose our color pairs */
 	init_pair(GREEN,  COLOR_GREEN,   COLOR_BLACK);
 	init_pair(YELLOW, COLOR_YELLOW,  COLOR_BLACK);
 	init_pair(BLUE,   COLOR_BLUE,    COLOR_BLACK);
@@ -1263,22 +1275,16 @@ int main(int argc, char *argv[])
 	/* Init the scale */
 	init_display_scale(console_width);
 	
-	pthread_create(&kb_dt, NULL, (void *)&keyboard_thread, (void *) meterec);
+	pthread_create(&kb_dt, NULL, keyboard_thread, (void *) meterec);
 	
 	find_existing_takes(meterec);
 	
 	/* Start threads doing disk accesses */
-	if (meterec->record_cmd==START) {
-		
+	if (meterec->record_cmd==START)
 		start_record(meterec);
+	
+	if (meterec->playback_cmd==START)
 		roll(meterec);
-		
-	}
-	else if (meterec->playback_cmd==START) {
-		
-		roll(meterec);
-		
-	}
 	
 	/* Register the cleanup function to be called when C-c */
 	signal(SIGINT, halt);
