@@ -2,7 +2,7 @@
 
   meterec.h 
   Console based multi track digital peak meter and recorder for JACK
-  Copyright (C) 2010 2011 Fabrice Lebas
+  Copyright (C) 2009-2013 Fabrice Lebas
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -35,10 +35,10 @@
 #define MAX_TAKES 100
 
 /* size of disk wait buffer, must be power of two */
-#define DISK_SIZE 131072
+#define DBUF_SIZE 0x20000 
 
 /* size of disk buffers */
-#define BUF_SIZE 4096
+#define ZBUF_SIZE 4096
 
 /*number of seek indexes*/
 #define MAX_INDEX 12
@@ -85,6 +85,12 @@
 #define CON 0
 #define CON_OUT (1)
 
+/* loop boundaries */
+#define BOUND_LOW 1
+#define BOUND_HIGH 2
+#define BOUND_ALL 3
+
+
 #define MAX_UINT ((unsigned int)(-1))
 
 /*
@@ -314,4 +320,4 @@ void start_record(struct meterec_s *meterec) ;
 void cancel_record(struct meterec_s *meterec) ;
 
 int set_loop(struct meterec_s *meterec, unsigned int loophead);
-void clr_loop(struct meterec_s *meterec);
+void clr_loop(struct meterec_s *meterec, unsigned int bound);
